require 'upkey/chatterly/accessor'
require 'upkey/chatterly/response'

require 'upkey/chatterly/urls/accessor'
require 'upkey/chatterly/urls/builder'
require 'upkey/chatterly/urls/response'

require 'upkey/chatterly/alerts/missing_argument'
require 'upkey/chatterly/errors/chatterly_argument_error'
require 'upkey/chatterly/errors/chatterly_error'

require 'faraday'
require 'json'

module Upkey
  module Chatterly
    class Client
      attr_reader :access_key, :chatterly_url, :conn

      def initialize(args = {})
        MissingArgument.alert unless args[:access_key] && args[:chatterly_url]

        @access_key = args[:access_key]
        @chatterly_url = args[:chatterly_url]
        @conn = set_up_faraday
      end

      def respond(args = {})
        key = args[:key]
        id = args[:id]

        message = Upkey::Chatterly::Accessor.get_message(self, key, id)
        resp = Upkey::Chatterly::Accessor.get_resp(self, message.body)

        Upkey::Chatterly::Response.new(message, resp)
      end

      def method_missing(*args)
        case args[0]
        when :post_url
          Upkey::Chatterly::Urls::Builder.post_url(args[1], self)
        when :get_url
          Upkey::Chatterly::Urls::Accessor.get_url(args[1], self)
        else
          raise NoMethodError
        end
      end

      private

      def set_up_faraday
        Faraday.new(:url => @chatterly_url) do |faraday|
          faraday.request  :url_encoded
          faraday.response :logger
          faraday.adapter  Faraday.default_adapter
        end
      end
    end
  end
end
